﻿using System;
using System.Linq;
using System.Reflection;
using System.Text;

namespace TaskGenerater
{
    public class Generater
    {
        private string file;
        private Assembly assembly;
        private Type[] allTypes;
        private Type clientType;
        private Type channelType;

        public string Namespace { get; private set; }
        public string ClientName { get; private set; }
        public string MethodsString { get; private set; }

        public static string Generate(string directory)
        {
            var files = Helper.SerachFile(directory, "Reference.cs");
            if (files.Any() == false)
                throw new MyException("当前项目找不到服务引用!");

            var sb = new StringBuilder();
            foreach (var file in files)
            {
                sb.Append(GenerateOne(file));
                sb.AppendLine();
            }
            return Templete.MergeFile(sb.ToString());
        }

        public static string GenerateOne(string file)
        {
            var taskGenerate = new Generater(file);
            return taskGenerate.Generate();
        }

        private Generater(string filePath)
        {
            this.file = filePath;
        }

        private string Generate()
        {
            assembly = Helper.CompileToAssembly(file);
            allTypes = assembly.GetTypes();
            clientType = allTypes.FirstOrDefault(t => t.Name.Contains("Client") && t.BaseType.Name.Contains("ClientBase"));
            channelType = allTypes.FirstOrDefault(t => t.Name.Contains("ClientChannel"));
            if (channelType == null)
                throw new MyException("找不到ClientChannel类!");

            if (clientType == null)
                throw new MyException("找不到Client类!");

            this.Namespace = clientType.Namespace;
            this.ClientName = clientType.Name;
            this.MethodsString = GenerateMethods();
            return Templete.MergeClass(this);
        }

        private string GenerateMethods()
        {
            var sb = new StringBuilder();
            var channelMethods = channelType.GetMethods();
            foreach (var beginMethod in channelMethods.Where(m => m.Name.StartsWith("Begin")))
            {
                var endMethodName = "End" + beginMethod.Name.Substring(5);
                var endMethod = channelMethods.FirstOrDefault(m => m.Name == endMethodName);
                if (endMethod == null)
                    throw new MyException(string.Format("找不到:{0}方法!", endMethodName));

                var oneMethodString = Templete.MergeMethod(GetMethodData(beginMethod, endMethod));
                sb.Append(oneMethodString);
                sb.AppendLine();
            }
            return sb.ToString();
        }

        private dynamic GetMethodData(MethodInfo beginMethod, MethodInfo endMethod)
        {
            dynamic result = new System.Dynamic.ExpandoObject();
            var methodNameBase = beginMethod.Name.Substring(5);
            var parameters = beginMethod.GetParameters().Where(p => p.Name != "callback" && p.Name != "asyncState");

            result.ReturnType = string.Empty;
            result.ParamsNames = string.Empty;
            result.ParamsTypeAndNames = string.Empty;
            result.MethodName = methodNameBase + "TaskAsync";
            result.BeginMethod = beginMethod.Name;
            result.EndMethod = endMethod.Name;

            if (endMethod.ReturnType.Name != "Void")
            {
                var temp = endMethod.ReturnType.IsGenericType ? GetGenericTypeString(endMethod.ReturnType) : endMethod.ReturnType.Name;
                result.ReturnType = string.Format("<{0}>", temp);
            }
            if (parameters.Any())
            {
                result.ParamsTypeAndNames = string.Join(",", parameters.Select(p =>
                {
                    var typeName = p.ParameterType.IsGenericType ? GetGenericTypeString(p.ParameterType) : p.ParameterType.Name;
                    return string.Format("{0} {1}", typeName, p.Name);
                }));
                result.ParamsNames = "," + string.Join(",", parameters.Select(p => { return p.Name; }));
            }
            return result;
        }

        private string GetGenericTypeString(Type returnType)
        {
            if (returnType.IsGenericType == false)
                return string.Empty;

            var genericTypeName = returnType.GetGenericTypeDefinition().Name;
            genericTypeName = genericTypeName.Substring(0, genericTypeName.LastIndexOf('`'));
            var genericArguments = returnType.GetGenericArguments().Select(a => a.Name);
            return string.Format("{0}<{1}>", genericTypeName, string.Join(",", genericArguments));
        }
    }
}